/**************** Foreign-Character Macros **************************

  For TSE (The Semware Editor), pre-release version 1.0

  Author:  David Mayerovitch
           7 June 1993

  Description:

  This pair of macros provides a quick and convenient way of entering
  foreign-language and other special characters.

  mSelectLanguage() presents a menu of languages or other
  special-character sets for the user to choose from. I have assigned
  this macro to <Alt `>.

  mAccenter() enables the entry of the special characters belonging to
  the selected set.  I have assigned this macro to the key <`>. Call
  this the trigger key.

  These macros do what you do when writing foreign characters by hand:
  first write the basic letterform, then add the diacritical mark.

  Example: If the selected language is French and you want to enter
  the character <> (e with accent aigu): type the basic character
  <e>, then press the trigger key.  A menu appears:

  Ŀ
   Aigu    
   Grave   
   Circ    
  

  The hotkeys A, G, and C generate the three possible e-based accented
  characters in French. Press <A> and the <e> just typed will be
  replaced with <>.

  When there is only one possible accented version of the character
  just typed (e.g.  in German), the macro automatically inserts it
  without presenting a menu.

  The macros as presented here offer the following character sets:
  French:           
  German:     
  International (German + French):                
  Currency: c/C -->  (cents)  l/L -->  (pounds)  y/Y -->  (yen)

  You may easily edit the macros to provide any substitutions
  you want, arranged in languages or character sets of your
  choice.  Strings as well as single characters may be
  substituted.

  If a printable key such as <`> is chosen as the trigger for
  mAccenter(), the Literal() command (assigned to <Ctrl P> in
  the factory configuration of TSE) may be used whenever the
  character itself is required in the text.

  Comments and improvements are welcome.

********************************************************************/

// global variable:
integer language = 1 // French is default

// LANGUAGE SELECTION ROUTINES:

menu langMenu()
  history
  "&French"  "&German"   "&International"
  "", , Divide  "&Currency"
end

proc mSelectLanguage()
  langMenu()
  language = MenuOption()
end

// CHARACTER SUBSTITUTION ROUTINES:

proc sub(string newstring)
// replaces current char by newstring, forcing insert.
  DelChar()
  InsertText(newstring,_insert_)
end

menu FrenchAMenu()
  "&Grave ", sub('') "&Circ  ", sub('')
end

menu IntlAMenu()
  "&Grave  ", sub('')
  "&Circ   ", sub('') "&Umlaut ", sub('')
end

menu IntlOMenu()
  "&Circ   ", sub('') "&Umlaut ", sub('')
end

menu IntlUMenu()
  "&Circ   ", sub('') "&Umlaut ", sub('')
end

menu FrenchEMenu()
  "&Aigu ", sub('') "&Grave ", sub('') "&Circ ", sub('')
end

proc French()
  case Chr(CurrChar())
    when 'a' FrenchAMenu() when 'e' FrenchEMenu()
    when 'E' sub('') when 'c' sub('') when 'C' sub('')
    when 'i' sub('') when 'o' sub('') when 'u' sub('')
  otherwise Right()
    endcase
end

proc German()
  case Chr(CurrChar())
    when 'a' sub('') when 'A' sub('') when 'o' sub('')
    when 'O' sub('') when 'u' sub('') when 'U' sub('')
  otherwise Right()
    endcase
end

proc International()
  case Chr(CurrChar())
    when 'e' FrenchEMenu() when 'a' IntlAMenu()
    when 'o' IntlOMenu()  when 'u' IntlUMenu()
    when 'A' sub('') when 'c' sub('') when 'C' sub('')
    when 'i' sub('') when 'E' sub('') when 'O' sub('')
    when 'U' sub('')
  otherwise Right()
    endcase
end

proc Currency()
  case UpCase(Chr(CurrChar()))
    when 'C' sub('') // cents
    when 'L' sub('') // pounds
    when 'Y' sub('') // yen
  otherwise Right()
    endcase
end

proc mAccenter()
  Left()
  Case language
    when 1 French() when 2 German() when 3 International()
    when 5 Currency()
    endcase
end
<`> mAccenter()
<Alt `> mSelectLanguage()

/************************** End of macros **************************/

